function I=instance(file,varargin)
% Generate an instance from ballmodel -mode sedumi2 and create reduced
% dimensionality matrices for the protein

negtol=-1e-5;
zerodist=1e-1;
sqrt2=sqrt(2);
full=1;
bdelt_tol = 1e-2;

i=1;
while i<=length(varargin)
  switch varargin{i}
    case 'zerodist',zerodist=varargin{i+1}; i=i+1;
    otherwise
      fprintf('Bad option "%s", exiting\n',varargin{i});
      I=0; return;
  end
  i=i+1;
end

f=fopen(file,'r');
% Read header
flds=textscan(f,'%f %f %f %f \n',1);
n=flds{1};                              % No. of balls
nr=flds{2};                             % Number of residues
m=flds{3};                              % No. constraints
encd=flds{4};                           % Encoding dimension

% Read encoding string
encstr = fgetl(f);

% Read ball info
flds=textscan(f,'%f %f %f %f %f %f',n);
res=[flds{1} flds{2}]';
T=center([flds{3} flds{4} flds{5}]');
radii=flds{6}';

% Read constraint info
flds=textscan(f,'%f %f %f %f',m,'commentStyle','#');
ci=flds{1};
cj=flds{2};
b=flds{3}.^2;
iseq=logical(flds{4});

tn=triang(n);
tn1=triang(n-1);                        % Used for the number of dist constraints
sz=[n n];
nn=n*n;
n_eq=sum(iseq);
n_in=m-n_eq;

linidx=sub2ind(sz,ci,cj);
eqidx=linidx(iseq);
eqdist=b(iseq);
inidx=linidx(~iseq);
indist=b(~iseq);

% Full version
jdxs=([sub2ind(sz, ci, ci);           % Diag ci
       sub2ind(sz, cj, cj);           % Diag cj
       sub2ind(sz, ci, cj);           % Off-diag, upper triang
       sub2ind(sz, cj, ci)]);         % Off-diag, lower triang

o2tn1=(1:tn1)';
idxs=[o2tn1; o2tn1; o2tn1; o2tn1;];
tn1ones=ones(tn1,1);
vals=[tn1ones; tn1ones; -tn1ones; -tn1ones];
AS=sparse(idxs,jdxs,vals,m,nn);
AL=sparse(find(~iseq),1:n_in,-ones(n_in,1),m,n_in);

GT=T'*T;
xS=vec(GT);

resid=AS*xS-b;
xL=resid(~iseq);
x=[xL;xS];

% Make adjustments for feasibility and small distances->0
xold=x;
bold=b;
xL(xL<=zerodist)=0;
b=AS*xS+AL*xL;
x=[xL;xS];

% Info on nonzero zL
zLidx=find(xL==0);
nzL=length(zLidx);
Ibar=sparse(zLidx,1:nzL,ones(nzL,1),n_in,nzL);

% Add centering constraints
aa=sparse(n,n);
for ii=1:n
  aa(:)=0;
  aa(:,ii)=.5;
  aa(ii,:)=.5;
  aa(ii,ii)=1;
  AS(ii+m,:)=vec(aa);
end
AL=[AL; sp0(n,n_in)];
b=[b; zeros(n,1)];
bold=[bold; zeros(n,1)];
iseq(m+1:m+n)=1;
m=m+n;
n_eq=n_eq+n;

bdelt=norm(bold-b)/norm(bold);
if bdelt > bdelt_tol
  fprintf('Warning: %s has large b change: %g\n',file,bdelt);
end

[q ll]=eig(GT);
l=diag(ll);
if find(l<negtol)
  fprintf('Warning: Gram matrix is not PSD\n');
end

% Form QQ, do not include the 3 nonzero eigs
QQ(:,i) = vec(q(:,i)*q(:,i)');

% Features
frmt='';
for i=1:encd, frmt=[frmt '%f ']; end
flds=textscan(f,frmt,n,'CommentStyle','#');
F=cell2mat(flds)';                      % Transpose for better eff on outer products
fclose(f);

F=sparse(F);

% fprintf('Warning: removing nonzero features\n');
% nnzcols=find(sum(abs(F),2));
% F=F(nnzcols,:);

FF=linear_mapR(F,ci,cj,iseq);
kdim=size(FF,2);

% Map to visualize a matrix of cL
cLmap=sub2ind([n n],ci(~iseq),cj(~iseq));

% Set up index map to put sc close to main chain
mcscmap = zeros(2,nr);
mcscmap(sub2ind([2 nr],res(2,:)+1,res(1,:))) = 1:n;
mcscmap = mcscmap(mcscmap>0);

I = struct(...
  'nb',n,...
  'nr',nr,...
  'kdim',kdim,...
  'tn',tn,...
  'm',m,...
  'n_in',n_in,...
  'nzL',nzL,...
  'xL',xL,...
  'x',x,...
  'FF',FF,...
  'Ibar',Ibar,...
  'b',b,...
  'encstr',encstr,...
  'QQ',QQ,...
  'AL',AL,...
  'AS',AS,...
  'cLmap',cLmap,...
  'mcscmap',mcscmap,...
  'res',res,...
  'full',full,...
  'iseq',iseq,...
  'linidx',linidx,...
  'eqidx',eqidx,...
  'eqdist',eqdist,...
  'inidx',inidx,...
  'indist',indist,...
  'ci', ci,...
  'cj', cj,...
  'F',F...
);

I.A=[AL AS];
I.K.l=n_in;
I.K.s=n;
I.T = T;
I.xS = xS;


end

function FF=linear_mapR(F,ci,cj,iseq)
% Create the map on features by parameters to linear objective,
% reduced dimensionality. 
ci=ci(~iseq);
cj=cj(~iseq);
nlin=length(ci);
for idx=1:nlin
  i=ci(idx); 
  j=cj(idx);
  tmp=F(:,i)*F(:,j)';
  FF(:,idx)=mat2svec(tmp+tmp');
end
FF=FF';
end



% function FF=linear_mapF(F,ci,cj,iseq)
% % Create the map on features by parameters to linear objective,
% % full dimensionality. 
% ci=ci(~iseq);
% cj=cj(~iseq);
% nlin=length(ci);
% for idx=1:nlin
%   i=ci(idx); 
%   j=cj(idx);
%   tmp=F(:,i)*F(:,j)';
%   FF(:,idx)=vec(tmp+tmp');
% end
% FF=FF';
% end
