function R=obj_design_fixed(file,varargin)

% function R=obj_design_fixed(file,varargin)
% 
% Design objectives for given file in domo format or an instance.  The
% code functions under the assumption that the objective is based on
% the linear part of the program and is unique under norm
% minimization.

% Variable ordering: 
% --------------------------
% cL y zLbar omega 
% --------------------------

% Equations:

% OML: cL
% DFL: ALin'*yin + Ibar*zLbar - cL = 0
% DFS: ASeq'*yeq + ASin'*yin + QQ*omega = 0
% NZS: 1'*omega = nb-3
% OEL: beq'*yeq + bin'*yin - cL = 0  [3]
% OFL: cL(1) = 1

startime=cputime();

solver = 'lsqlin';
min_eqns = {'OML'};
eql_eqns = {'DFL','DFS','NZS'};
getvars = 1;
savecL = 0;

eql_min = 0;
eql_wgt = 50;

i=1;
while i<=length(varargin)
  switch varargin{i}
    case 'solver',
      solver=varargin{i+1}; i=i+1;
    case 'min_eqns',
      min_eqns=varargin{i+1}; i=i+1;
    case 'eql_eqns',
      eql_eqns=varargin{i+1}; i=i+1;
    case 'eql_min', eql_min=1;
    case 'min_oel',
      min_eqns = {min_eqns{:}, 'OEL'};
    case 'eql_oel',
      eql_eqns = {eql_eqns{:}, 'OEL'};
    case 'eql_wgt',
      eql_wgt=varargin{i+1}; i=i+1;
    case 'savecL'
      savecL = 1;
    otherwise
      fprintf('**ERROR: Unknown option %s, Exiting\n',varargin{i}); 
      return;
  end
  i=i+1;
end

switch solver
  case {'sedumi','lsqlin'}, [];
  otherwise
    fprintf('**ERROR: Unknown solver %s, Exiting\n',varargin{i}); 
    return;
end

fprintf('Objective Design, solver: %s  eql_min: %d  eql_wgt: %g\n',...
        solver,eql_min,eql_wgt);
fprintf('Min_Eqns: ');   fprintf('%s ',min_eqns{:});
fprintf('\nEql_Eqns: '); fprintf('%s ',eql_eqns{:});
fprintf('\n');

tic;

if isstr(file)
  fprintf('Processing %s\n',file);
  I = instance(file);
  I = reduce_instance(I);
else
  I = file;
end


nl  = I.n_in;
m   = I.m;
nb3 = I.nb-3;
tn  = I.tn;
nzL = I.nzL;
ncL = nl;

% DFL
DFL.f = [-spI(ncL) I.AL'];
DFL.l=[I.Ibar sp0(nl,nb3)];
DFL.b = sp0(nl,1);

% DFS
DFS.f = [sp0(tn,ncL) I.AS'];
DFS.l = [sp0(tn,nzL) I.QQ];
DFS.b = sp0(tn,1);

% NZS
NZS.f = sp0(1,ncL+m);
NZS.l = [sp0(1,nzL) ones(1,nb3)];
NZS.b = tn; %(nb3+3)^2;                      % Get the scaling right, either will do

% OEL
OEL.f = [-I.xL'  I.b'];
OEL.l = sp0(1,nzL+nb3);
OEL.b = sp0(1,1);

% OML
OML.f = [spI(ncL) sp0(ncL,m)];
OML.l = sp0(ncL,nzL+nb3);
OML.b = sp0(ncL,1);

% OFL
OFL.f = [sparse(1,1,1,1,ncL) sp0(1,m)];
OFL.l = sp0(1,nzL+nb3);
OFL.b = [1;];

% Easy access structure
eqns=struct(...
  'DFL',DFL, ...
  'DFS',DFS, ...
  'NZS',NZS, ...
  'OEL',OEL, ...
  'OML',OML, ...
  'OFL',OFL  ...
  );

% Equations for minimization
DM.f=[];
DM.l=[];
DM.b=[];
for i=1:length(min_eqns)
  min_eqns{i};
  eqn=eqns.(min_eqns{i});
  DM.f = [DM.f; eqn.f];
  DM.l = [DM.l; eqn.l];
  DM.b = [DM.b; eqn.b];
end

% Equations for equality constraints
DE.f=[];
DE.l=[];
DE.b=[];
for i=1:length(eql_eqns)
  eqn=eqns.(eql_eqns{i});
  DE.f = [DE.f; eqn.f];
  DE.l = [DE.l; eqn.l];
  DE.b = [DE.b; eqn.b];
end
nmin   = size(DM.f,1);
neql   = size(DE.f,1);
linear = nb3 + nzL;
free   = ncL + m;

totmin  = sum(nmin);
toteql  = sum(neql);
nfree   = sum(free);
nlinear = sum(linear);

setuptime=toc;

fprintf('Setup Complete\n');
fprintf('D%1d> %8s %8s %8s %8s %8s %8s\n',...
        0,'toteqn','totvar','totmin','toteql','nfree','nlinear');
fprintf('D%1d> %8g %8g %8g %8g %8g %8g\n',...
        1, totmin+toteql, nfree+nlinear, totmin, toteql, nfree, nlinear)

min_eqns = min_eqns;
eql_eqns = eql_eqns;
totmin = totmin;
toteql = toteql;
nlinear = nlinear;
nfree = nfree;

switch solver
  case 'sedumi'
    fprintf('Solving SOCP formulation with sedumi\n');
    nsocp = [];
    D = [DM.f DM.l 
         DE.f DE.l];
    [dm dn]=size(D);
    Dobj = sparse(dn,1);
    if totmin>0                         % Add slacks for minimization
      D=[D [sp0(totmin,1) spI(totmin,totmin)  
            sp0(toteql,1) sp0(toteql,totmin)]];
      Dobj = sparse(dn+1,1,1,size(D,2),1);
      nsocp = [totmin+1];
    end
    [dm dn]=size(D);
    if eql_min                          % Allow slacks for equalites, weight their min
      D=[D [sp0(totmin,1) sp0(totmin,toteql)
            sp0(toteql,1) spI(toteql,toteql)]];
      [ii jj vv]=find(Dobj);
      Dobj = sparse([ii dn+1], [jj 1], [vv eql_wgt], size(D,2), 1);
      nsocp = [nsocp toteql+1];
    end
    Db=[DM.b;  DE.b];
    KD=struct('f',nfree,'l',nlinear,'q',nsocp);
    pars.fid=0;
    pars.solver = 'sedumi';
    [prim dual dinfo]=sdpsolver(D,Db,Dobj,KD,pars);
    objval  = full(Dobj'*prim);
    slacks=prim(nfree+nlinear+1 : length(prim));
    prim = prim(1:nfree+nlinear);
             
  case 'lsqlin'
    fprintf('Solving NN-LSQ formulation with lsqlin, eql_wgt: %g\n',eql_wgt);
    D1 = [DM.f   DM.l];
    D2 = [DE.f   DE.l];
    D  = [D1;    eql_wgt*D2];
    Db = [DM.b;  eql_wgt*DE.b ];
    [md nd] = size(D);
    if md <= nd
      fprintf('ERROR: Cowardly refusal to deal with underdetermined system\n');
      return;
    end
    lb = repmat(-Inf,nd,1);
    ub = repmat( Inf,nd,1);
    lb(nd-nlinear+1:nd) = 0;
    tic;
    [prim objval slacks flag dinfo] = lsqlin(D,Db,[],[],[],[],lb,ub);
    dinfo.cpusec = toc;
    dinfo.flag = flag;
    dinfo.msg = lsqlin_msg(flag);
    
  otherwise
    fprintf('**ERROR: Unknown solver %s, Exiting\n',varargin{i}); 
    return;
end

totaltime = cputime()-startime;

cL    = prim(1:ncL);
y     = prim(ncL+1:ncL+m);
zL    = prim(ncL+m+1:ncL+m+nzL);
omega = prim(ncL+m+nzL+1:ncL+m+nzL+nb3);

fprintf('D%1d> %8s %8s %8s\n',...
        2,'objval','Minutes','Smsg');
fprintf('D%1d> %8.2e %8.2f %8s\n',...
        3,objval, totaltime/60, dinfo.msg);

if savecL
  if isstr(file)
    base = parse_filename(file);
  else
    base = 'obj';  
  end
  save([base '.cL'],'cL','-ASCII','-DOUBLE');
end
  

env2struct('R',who());
end

function msg=lsqlin_msg(flag)

switch flag
  case  1, msg = '';
  case  3, msg = 'SmResid';
  case  0, msg = 'MaxIter';
  case -2, msg = 'Infeas';
  case -4, msg = 'IllCond';
  case -7, msg = 'Progr';
end

end
